/*
 * thread.C,v 1.10 1993/12/02 14:52:45 lcp Exp
 */


    /*********************************************************************
    Copyright (C) 1993 by Citibank, N.A., all rights reserved.

    Permission to use, copy, modify and distribute this software and its 
    documentation for any purpose and without fee, except possible 
    redistribution costs, is hereby granted, provided that (1) the above 
    copyright notice appears in all copies and (2) both that copyright-
    notice and this permission-notice appear in supporting documentation and 
    (3) the name of Citibank, N.A. is not used in advertising or publicity 
    pertaining to distribution of the software without specific and written 
    prior permission of Citibank, N.A.

    The Software is provided "as is."  Citibank, N.A. disclaims all 
    warranties, express or implied, including but not limited to warranties 
    of suitability, merchantability and fitness for any purpose.  In no 
    event shall Citibank, N.A. be responsible for any loss of use, data or 
    profit or for any other special, indirect, incidental or consequential 
    damages, neither in an action of contract nor in an action of tort, 
    arising out of or in connection with the use or performance of this 
    software.
    *********************************************************************/


#include <objdce/thread.H>
#include <objdce/objtranparams.H>
#include <stdio.h>
#ifndef _NTWIN
#include <objdce/error.H>
#endif 
#include <objdce/defs.H>
#include <objdce/dcecheckstatus.H>

#ifndef _NTWIN 

#ifdef DOS
#include <dce/cmaerrno.h>
#else
#if HAVE_CMA_ERRNO_H
#include <dce/cma_errno.h>
#endif
#endif

#endif // _NTWIN

static void* AsyncRunner(void* obj)

{
  if (pthread_setcancel(CANCEL_OFF) < 0)
    CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't set thread cancel state");
  if (pthread_setasynccancel(CANCEL_OFF) < 0)
    CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't set thread async cancel stats");
  void *status = ((ThreadObject*)obj)->Run();
  return (status);
}

ThreadObject::ThreadObject()

{
}

ThreadObject::~ThreadObject()

{
}

void ThreadObject::SetTran(Transaction* t)

{
    parentTran = t;
}

void ThreadObject::YieldMe()

{
    pthread_yield();
}

void ThreadObject::SleepMe(const long secs, const long uSecs)

{
    struct timespec sleepTime;
    sleepTime.tv_sec = secs;
    sleepTime.tv_nsec = uSecs * 1000;
    if (pthread_delay_np(&sleepTime))
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't sleep thread");
}

void ThreadObject::ExitMe(void* retVal)

{
    pthread_exit(retVal);
}

void ThreadObject::SetKillable(const Boolean on)

{
  if (on)
    {
      if (pthread_setcancel(CANCEL_ON) < 0)
	CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't set thread cancel state");
    }
  else
    {
      if (pthread_setcancel(CANCEL_OFF) < 0)
	CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't set thread cancel state");
    }
}

void ThreadObject::DoKill()

{
    pthread_testcancel();
}

AsyncThread::AsyncThread(ThreadObject& tObj)

{
  attrUsed = false;
  Init(AsyncRunner, (void*)(&tObj));
}

AsyncThread::AsyncThread(ThreadObject& tObj, long stackSize)

{
  attrUsed = true;
  if (pthread_attr_create(&threadAttr))
    CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't create thread attributes");
  if (pthread_attr_setstacksize(&threadAttr, stackSize))
    CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't set thread attributes");
  Init(AsyncRunner, (void*)(&tObj));
}

AsyncThread::AsyncThread()

{
  attrUsed = false;
}

void AsyncThread::Init(pthread_startroutine_t func , void* args)

{
    int err;
    detached = done = false;
    if (!attrUsed)
      err = pthread_create(&threadRep, pthread_attr_default, func, args);
    else
      err = pthread_create(&threadRep, threadAttr, func, args);
    if (err)
    {
      // Abort the program if this fails
      CheckStatus(errno, true, ErrorLevels::ErrorMsg, "Couldn't spawn a new thread");
    }
}    

AsyncThread::~AsyncThread()

{
  if (attrUsed)		// OK even if the thread is still running
    {
      if (pthread_attr_delete(&threadAttr))
	CheckStatus(errno, false, ErrorLevels::ErrorMsg,"Couldn't delete thread attributes");
    }
  if (!done && !detached)
    {
      if (pthread_detach(&threadRep))
	CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't detach from thread");
    }
}

void* AsyncThread::Wait()

{
    if (detached)
    {
    ObjtranReport.Report(ErrorLevels::InfoMsg, "Waiting for a detached thread is not allowed");
    return (void *)nil;
    }
    else
    {
	void* retVal;
	if (pthread_join(threadRep, &retVal))
	  CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't join with thread");
	done = true;
	return retVal;
    }
}

void AsyncThread::Detach()

{
    if (pthread_detach(&threadRep))
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't detach from thread");
    detached = true;
}

void AsyncThread::SetPriority(const unsigned long newPrior)

{
  if (pthread_setprio(threadRep, (int)newPrior))
    CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't set thread priority");
}

unsigned long AsyncThread::GetPriority() const

{
    unsigned long prior = 0;
    prior = pthread_getprio(threadRep);
    if (prior < 0)
      CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't get thread priority");
    return prior;
}

void AsyncThread::Kill()

{
  if (pthread_cancel(threadRep))
    CheckStatus(errno, false, ErrorLevels::ErrorMsg, "Couldn't cancel thread");
  done = true;
}

AsyncThread::operator void*()

{
    return Wait();
}

