/*
 * sampleServer.C:
 *    This server implements the RPC interface in sample.idl. The CallMe
 * function doesn't really do anything. 
 *
 * sampleServer.C,v 1.4 1993/03/31 17:39:26 gjb Exp
 */

#include <objdce/statics.H>
#include <iostream.h>
#include <objdce/nontranserver.H>
#include <objdce/binding.H>
#include <objdce/error.H>
#include <proxy.h>
#include <String.h>
#include <iostream.h>
#include <stdlib.h>


error_status_t RpcNsBindingImportBegin_(handle_t handle,
					unsigned long syntax_name,
                                        unsigned char * entry_name,
					unsigned char * stringUuid,
                                        unsigned long *imp_context)
{
	rpc_ns_handle_t import_context;
	error_status_t status;
	uuid_t *ObjUuid = NULL;


	if (*stringUuid != '\0') {
		printf("uuid_from_string being called\n");
		ObjUuid = (uuid_t *) malloc(sizeof(uuid_t));
		uuid_from_string(stringUuid, ObjUuid, &status);       
	}

	printf("RpcNsBindingImportBegin called\n");
        rpc_ns_binding_import_begin(syntax_name, 
	                            entry_name,
				    NULL,
				    ObjUuid, 
				    &import_context,
	    		     	    &status);
	printf("RpcNsBindingImportBegin returned with status = %d and import_context = 0x%x\n",
			status, import_context);
					     
	 if (ObjUuid)
		free(ObjUuid);

	 *imp_context = (unsigned long) import_context;	
	 return status;

}


error_status_t   RpcNsBindingImportNext_(handle_t handle,
					 unsigned long imp_context,
					 long *length,
					 unsigned char *binding)
{
          error_status_t status;
	  unsigned_char_t *string_binding;
	  rpc_binding_handle_t binding_handle;

          rpc_ns_handle_t import_context =  (rpc_ns_handle_t) imp_context;
	  
	  printf("RpcNsBindingImportNext called with import_context = 0x%x\n", import_context);

	  rpc_ns_binding_import_next(import_context,
				     &binding_handle,
				     &status);

	  printf("RpcNsBindingImportNext returned with status %d \n", status);

	  switch (status) {
	     case rpc_s_ok:
	      rpc_binding_to_string_binding(binding_handle,
					    &string_binding,
					    &status);
	      if (status != rpc_s_ok)
			return status;
              strcpy((char *)binding, (char *)string_binding);
	      *length = strlen((char *)binding);
	      rpc_binding_free(&binding_handle, &status); 
	      rpc_string_free(&string_binding, &status);
	      fprintf(stderr, "String binding = %s\n", binding);
	      return status;

             default:
	       return status;
          }
}

error_status_t RpcMgmtIsServerListening_(handle_t handle,
					unsigned char *stringBinding)
{
	error_status_t status;
	rpc_binding_handle_t bindingHandle;

	fprintf(stderr, "RpcMgmtIsServerListening called\n");
	rpc_binding_from_string_binding(stringBinding, &bindingHandle, &status);
	if (status != rpc_s_ok)
		return status;
	rpc_mgmt_is_server_listening(bindingHandle, &status);
	rpc_binding_free(&bindingHandle, &status);

	printf("RpcMgmtIsServerListening returned with status %d\n", status);
	return status;

	
}

error_status_t RpcMgmtStopServerListening_(handle_t handle,
					   unsigned char *stringBinding)
{
	error_status_t status;
	rpc_binding_handle_t bindingHandle;

	printf("RpcMgmtStopServerListening called\n");

	rpc_binding_from_string_binding(stringBinding, &bindingHandle, &status);
	if (status != rpc_s_ok)
		return status;   
	rpc_mgmt_stop_server_listening(bindingHandle, &status); 
	rpc_binding_free(&bindingHandle, &status);

	printf("RpcMgmtStopServerListening returned with status %d\n", status);
	return status;	

}
error_status_t RpcNsBindingImportDone_(handle_t handle,
				       unsigned long imp_context)
{
	rpc_ns_handle_t import_context =  (rpc_ns_handle_t) imp_context;	
	error_status_t status;

	printf("RpcNsBindingImportDone called\n");

	rpc_ns_binding_import_done(&import_context, &status);
  	
	printf("RpcNsBindingImportDone returned with status %d\n", status);
	return status;
}
				       

error_status_t CallMe(handle_t hand, unsigned long argument)
{
	fprintf(stderr,"We are in CallMe\n");
	return 0;
}

error_status_t NullRpc(handle_t hand)
{
	fprintf(stderr, "NULLRPC\n");
	return 0;
}


static void PrintUsage()

{
  cerr << "Usage: proxyServer <server name>" << endl;
  cerr << "       proxyServer -wk <endpoint>" << endl;
}

main(int argc, char** argv)

{
  SetObjtranErrorLevel(DebugMsg);
  if ((argc < 2) || (argc > 3))
    {
      PrintUsage();
      return -1;
    }
  String serverName = argv[1];
  NonTranServer* server;
  if ((argc == 3) && !strcmp(argv[1], "-wk"))
  {
    int endpoint = atoi(argv[2]);
    server = new NonTranServer(RPCServer(Proxy,1,0), endpoint, 
                               TCP, (ServerLoginContext*)0);
			       
    cout << "Registered server at endpoint " << endpoint;
    cout << " and listening..." << endl;
  }
  else if (argc == 2)
  {
    cout << "Creating a server object" << endl;

    server = new NonTranServer(RPCServer(Proxy,1,0), serverName,
			       "Sample DCE server", TCP, 
			       (ServerLoginContext*)0);
    cout << "Registered server at " << serverName;
    cout << " and listening for RPC calls..." << endl;
  }
  server->StartServer();
  cout << "Server terminating..." << endl;
  delete server;
  return 0;
}
